const objectHash = require('object-hash');

const verifySignature = require('../util/verify-signature');
const { forum } = require('../util/db');

const read = async (hash) => {
  // Fetch content
  const data = await forum.get(hash);

  data.embeddedData = data.embeddedData || undefined;

  const {
    authors, content, signature, embeddedData, citations,
  } = data;

  // Verify hash
  const derivedHash = objectHash({
    authors, content, signature, embeddedData,
  });
  if (derivedHash !== hash) {
    throw new Error('hash mismatch');
  }

  if (signature) {
    // Verify signature
    if (!verifySignature(data)) {
      throw new Error('signature verificaition failed');
    }
  }

  return {
    authors, content, signature, embeddedData, citations,
  };
};

module.exports = read;
