const objectHash = require('object-hash');

const verifySignature = require('../util/verify-signature');
const { forum } = require('../util/db');

const write = async ({
  authors, content, citations, embeddedData, signature,
}) => {
  if (signature) {
  // Check author signature
    if (!verifySignature({
      authors, content, signature, embeddedData,
    })) {
      const err = new Error();
      err.status = 403;
      err.message = 'Signature verification failed';
      throw err;
    }
  }

  // Compute content hash
  const data = {
    authors, content, signature, embeddedData, citations,
  };
  const hash = objectHash({
    authors, content, signature, embeddedData,
  });

  // Store content
  await forum.put(hash, data);

  // Return hash
  return { hash, data };
};

module.exports = write;
