const { proposals } = require('../util/contracts');
const read = require('../api/read');
const { sendNewProposalEvent } = require('../matrix-bot/outbound-queue');

// Subscribe to proposal events
const start = () => {
  console.log('registering proposal listener');
  proposals.on('NewProposal', async (proposalIndex) => {
    console.log('New Proposal, index', proposalIndex);

    const proposal = await proposals.proposals(proposalIndex);
    console.log('postId:', proposal.postId);

    // Read post from database
    try {
      const post = await read(proposal.postId);
      console.log('post.content:', post.content);

      // Send matrix room event
      let message = `Proposal ${proposalIndex}\n\n${post.content}`;
      if (post.embeddedData && Object.entries(post.embeddedData).length) {
        message += `\n\n${JSON.stringify(post.embeddedData, null, 2)}`;
      }

      // The outbound queue handles deduplication
      sendNewProposalEvent(proposalIndex, message);
    } catch (e) {
    // Post for proposal not found
      console.error(`error: post for proposal ${proposalIndex} not found`);
    }
  });
};

module.exports = {
  start,
};
