const { setTargetRoomId } = require('./outbound-queue');

const {
  appState,
  proposalEventIds,
} = require('../util/db');

const {
  BOT_INSTANCE_ID,
  ETH_NETWORK,
} = process.env;

const handleCommand = async (client, roomId, event) => {
  // Don't handle unhelpful events (ones that aren't text messages, are redacted, or sent by us)
  if (event.content?.msgtype !== 'm.text') return;
  if (event.sender === await client.getUserId()) return;

  const helloRegex = /^!hello\b/i;
  const targetRegex = /^!target (.*)\b/i;
  const proposalRegex = /\bprop(|osal) ([0-9]+)\b/i;

  const { body } = event.content;

  if (helloRegex.test(body)) {
    console.log(`!hello roomId ${roomId}`);
    await client.replyNotice(roomId, event, 'Hello world!');
  } else if (targetRegex.test(body)) {
    const [, instanceId] = targetRegex.exec(body);
    console.log(`!target roomId ${roomId} instanceId ${instanceId}`);
    if (instanceId === BOT_INSTANCE_ID) {
      setTargetRoomId(roomId);
      await appState.put('targetRoomId', roomId);
      await client.replyNotice(roomId, event, `Proposal events will be sent to this room for network ${ETH_NETWORK}`);
    }
  } else if (proposalRegex.test(body)) {
    const [, , proposalIndexStr] = proposalRegex.exec(body);
    const proposalIndex = parseInt(proposalIndexStr, 10);
    console.log(`mention of proposal ${proposalIndex} in roomId ${roomId}`);
    try {
      const proposalEventId = await proposalEventIds.get(proposalIndex);
      const proposalEventUri = `https://matrix.to/#/${roomId}/${proposalEventId}`;
      const content = {
        body: `Proposal ${proposalIndex}: ${proposalEventUri}`,
        msgtype: 'm.text',
      };
      if (event.content['m.relates_to']?.rel_type === 'm.thread') {
        content['m.relates_to'] = event.content['m.relates_to'];
      }
      await client.sendEvent(roomId, 'm.room.message', content);
    } catch (e) {
      // Not found
    }
  }
};

const registerCommands = (client) => {
  client.on('room.message', (roomId, event) => handleCommand(client, roomId, event));
};

module.exports = {
  registerCommands,
};
