const {
  AutojoinRoomsMixin,
  MatrixClient,
  RustSdkCryptoStorageProvider,
  SimpleFsStorageProvider,
} = require('matrix-bot-sdk');

const { registerCommands } = require('./commands');
const { registerRoomEventHandler } = require('./room-events');

const {
  MATRIX_HOMESERVER_URL,
  MATRIX_ACCESS_TOKEN,
  BOT_STORAGE_PATH,
  BOT_CRYPTO_STORAGE_PATH,
} = process.env;

const storageProvider = new SimpleFsStorageProvider(BOT_STORAGE_PATH);
const cryptoProvider = new RustSdkCryptoStorageProvider(BOT_CRYPTO_STORAGE_PATH);
let client;
let joinedRooms;

const { startOutboundQueue } = require('./outbound-queue');

const start = async () => {
  console.log('MATRIX_HOMESERVER_URL:', MATRIX_HOMESERVER_URL);
  client = new MatrixClient(
    MATRIX_HOMESERVER_URL,
    MATRIX_ACCESS_TOKEN,
    storageProvider,
    cryptoProvider,
  );

  // Automatically join a room to which we are invited
  AutojoinRoomsMixin.setupOnClient(client);

  joinedRooms = await client.getJoinedRooms();
  console.log('joined rooms:', joinedRooms);

  // Before we start the bot, register our command handler
  registerCommands(client);

  // Handler for custom events
  registerRoomEventHandler(client);

  client.start().then(() => {
    console.log('Bot started!');
    // Start the outbound queue
    startOutboundQueue(client);
  });
};

const getClient = () => client;

module.exports = {
  start,
  getClient,
};
