const fastq = require('fastq');

const {
  proposalEventIds,
} = require('../util/db');

let client;
let targetRoomId;

const setTargetRoomId = (roomId) => {
  targetRoomId = roomId;
};

const processOutboundQueue = async ({ type, ...args }) => {
  console.log('processing outbound queue item');
  if (!targetRoomId) return;
  switch (type) {
    case 'NewProposal': {
      const { proposalIndex, text } = args;
      try {
        await proposalEventIds.get(Number(proposalIndex));
        // If this doesn't throw, it means we already sent a message for this proposal
      } catch (e) {
        if (e.status === 404) {
          console.log('sending to room', targetRoomId, { text });
          const eventId = await client.sendText(targetRoomId, text);
          await proposalEventIds.put(Number(proposalIndex), eventId);
        }
      }
      break;
    }
    default:
  }
};

const outboundQueue = fastq(processOutboundQueue, 1);
// Pause until client is set
outboundQueue.pause();

const startOutboundQueue = (c) => {
  client = c;
  // Resume now that client is set
  outboundQueue.resume();
};

const sendNewProposalEvent = (proposalIndex, text) => {
  outboundQueue.push({ type: 'NewProposal', proposalIndex, text });
};

module.exports = {
  setTargetRoomId,
  outboundQueue,
  startOutboundQueue,
  sendNewProposalEvent,
};
