const { recoverPersonalSignature } = require('@metamask/eth-sig-util');

const {
  matrixUserToAuthorAddress,
  authorAddressToMatrixUser,
} = require('../util/db');

const handleRegisterIdentity = async (client, roomId, event) => {
  const { message, signature } = event.content;
  console.log('Received request to register identity');
  let account;
  try {
    account = recoverPersonalSignature({ data: message, signature });
  } catch (e) {
    console.log('error: failed to recover signature:', e.message);
  }
  if (account) {
    try {
      const authorAddress = await matrixUserToAuthorAddress.get(event.sender);
      if (account === authorAddress) {
        await client.sendNotice(roomId, `Matrix user ${event.sender} author address ${account} already registered`);
      } else {
        await client.sendNotice(roomId, `Matrix user ${event.sender} updated author address from ${authorAddress} to ${account}`);
      }
    } catch (e) {
      // Not found
      await client.sendNotice(roomId, `Matrix user ${event.sender} registered author address ${account}`);
    }
    await matrixUserToAuthorAddress.put(event.sender, account);
    await authorAddressToMatrixUser.put(account, event.sender);
  }
};

const registerRoomEventHandler = (client) => {
  client.on('room.event', (roomId, event) => {
    // Note that state events can also be sent down this listener too
    if (event.state_key !== undefined) return; // state event

    switch (event.type) {
      case 'io.dgov.identity.register':
        handleRegisterIdentity(client, roomId, event);
        break;
      default:
    }
  });
};

module.exports = {
  registerRoomEventHandler,
};
